"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Utils = void 0;
var _randombytes = _interopRequireDefault(require("randombytes"));
var _jsStellarBase = require("@digitalbits-blockchain/js-stellar-base");
var _errors = require("./errors");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }
function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }
function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _toArray(arr) { return _arrayWithHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableRest(); }
function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }
function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }
var Utils;
(function (_Utils) {
  function buildChallengeTx(serverKeypair, clientAccountID, homeDomain) {
    var timeout = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 300;
    var networkPassphrase = arguments.length > 4 ? arguments[4] : undefined;
    var webAuthDomain = arguments.length > 5 ? arguments[5] : undefined;
    var memo = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : null;
    var clientDomain = arguments.length > 7 && arguments[7] !== undefined ? arguments[7] : null;
    var clientSigningKey = arguments.length > 8 && arguments[8] !== undefined ? arguments[8] : null;
    if (clientAccountID.startsWith("M") && memo) {
      throw Error("memo cannot be used if clientAccountID is a muxed account");
    }
    var account = new _jsStellarBase.Account(serverKeypair.publicKey(), "-1");
    var now = Math.floor(Date.now() / 1000);
    var value = (0, _randombytes.default)(48).toString("base64");
    var builder = new _jsStellarBase.TransactionBuilder(account, {
      fee: _jsStellarBase.BASE_FEE,
      networkPassphrase: networkPassphrase,
      timebounds: {
        minTime: now,
        maxTime: now + timeout
      }
    }).addOperation(_jsStellarBase.Operation.manageData({
      name: "".concat(homeDomain, " auth"),
      value: value,
      source: clientAccountID
    })).addOperation(_jsStellarBase.Operation.manageData({
      name: "web_auth_domain",
      value: webAuthDomain,
      source: account.accountId()
    }));
    if (clientDomain) {
      if (!clientSigningKey) {
        throw Error("clientSigningKey is required if clientDomain is provided");
      }
      builder.addOperation(_jsStellarBase.Operation.manageData({
        name: "client_domain",
        value: clientDomain,
        source: clientSigningKey
      }));
    }
    if (memo) {
      builder.addMemo(_jsStellarBase.Memo.id(memo));
    }
    var transaction = builder.build();
    transaction.sign(serverKeypair);
    return transaction.toEnvelope().toXDR("base64").toString();
  }
  _Utils.buildChallengeTx = buildChallengeTx;
  function readChallengeTx(challengeTx, serverAccountID, networkPassphrase, homeDomains, webAuthDomain) {
    var _transaction$timeBoun;
    if (serverAccountID.startsWith("M")) {
      throw Error("Invalid serverAccountID: multiplexed accounts are not supported.");
    }
    var transaction;
    try {
      transaction = new _jsStellarBase.Transaction(challengeTx, networkPassphrase);
    } catch (_unused) {
      try {
        transaction = new _jsStellarBase.FeeBumpTransaction(challengeTx, networkPassphrase);
      } catch (_unused2) {
        throw new _errors.InvalidSep10ChallengeError("Invalid challenge: unable to deserialize challengeTx transaction string");
      }
      throw new _errors.InvalidSep10ChallengeError("Invalid challenge: expected a Transaction but received a FeeBumpTransaction");
    }
    var sequence = Number.parseInt(transaction.sequence, 10);
    if (sequence !== 0) {
      throw new _errors.InvalidSep10ChallengeError("The transaction sequence number should be zero");
    }
    if (transaction.source !== serverAccountID) {
      throw new _errors.InvalidSep10ChallengeError("The transaction source account is not equal to the server's account");
    }
    if (transaction.operations.length < 1) {
      throw new _errors.InvalidSep10ChallengeError("The transaction should contain at least one operation");
    }
    var _transaction$operatio = _toArray(transaction.operations),
      operation = _transaction$operatio[0],
      subsequentOperations = _transaction$operatio.slice(1);
    if (!operation.source) {
      throw new _errors.InvalidSep10ChallengeError("The transaction's operation should contain a source account");
    }
    var clientAccountID = operation.source;
    var memo = null;
    if (transaction.memo.type !== _jsStellarBase.MemoNone) {
      if (clientAccountID.startsWith("M")) {
        throw new _errors.InvalidSep10ChallengeError("The transaction has a memo but the client account ID is a muxed account");
      }
      if (transaction.memo.type !== _jsStellarBase.MemoID) {
        throw new _errors.InvalidSep10ChallengeError("The transaction's memo must be of type `id`");
      }
      memo = transaction.memo.value;
    }
    if (operation.type !== "manageData") {
      throw new _errors.InvalidSep10ChallengeError("The transaction's operation type should be 'manageData'");
    }
    if (transaction.timeBounds && Number.parseInt((_transaction$timeBoun = transaction.timeBounds) === null || _transaction$timeBoun === void 0 ? void 0 : _transaction$timeBoun.maxTime, 10) === _jsStellarBase.TimeoutInfinite) {
      throw new _errors.InvalidSep10ChallengeError("The transaction requires non-infinite timebounds");
    }
    if (!validateTimebounds(transaction, 60 * 5)) {
      throw new _errors.InvalidSep10ChallengeError("The transaction has expired");
    }
    if (operation.value === undefined) {
      throw new _errors.InvalidSep10ChallengeError("The transaction's operation values should not be null");
    }
    if (!operation.value) {
      throw new _errors.InvalidSep10ChallengeError("The transaction's operation value should not be null");
    }
    if (Buffer.from(operation.value.toString(), "base64").length !== 48) {
      throw new _errors.InvalidSep10ChallengeError("The transaction's operation value should be a 64 bytes base64 random string");
    }
    if (!homeDomains) {
      throw new _errors.InvalidSep10ChallengeError("Invalid homeDomains: a home domain must be provided for verification");
    }
    var matchedHomeDomain;
    if (typeof homeDomains === "string") {
      if ("".concat(homeDomains, " auth") === operation.name) {
        matchedHomeDomain = homeDomains;
      }
    } else if (Array.isArray(homeDomains)) {
      matchedHomeDomain = homeDomains.find(function (domain) {
        return "".concat(domain, " auth") === operation.name;
      });
    } else {
      throw new _errors.InvalidSep10ChallengeError("Invalid homeDomains: homeDomains type is ".concat(_typeof(homeDomains), " but should be a string or an array"));
    }
    if (!matchedHomeDomain) {
      throw new _errors.InvalidSep10ChallengeError("Invalid homeDomains: the transaction's operation key name does not match the expected home domain");
    }
    var _iterator = _createForOfIteratorHelper(subsequentOperations),
      _step;
    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var op = _step.value;
        if (op.type !== "manageData") {
          throw new _errors.InvalidSep10ChallengeError("The transaction has operations that are not of type 'manageData'");
        }
        if (op.source !== serverAccountID && op.name !== "client_domain") {
          throw new _errors.InvalidSep10ChallengeError("The transaction has operations that are unrecognized");
        }
        if (op.name === "web_auth_domain") {
          if (op.value === undefined) {
            throw new _errors.InvalidSep10ChallengeError("'web_auth_domain' operation value should not be null");
          }
          if (op.value.compare(Buffer.from(webAuthDomain))) {
            throw new _errors.InvalidSep10ChallengeError("'web_auth_domain' operation value does not match ".concat(webAuthDomain));
          }
        }
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
    if (!verifyTxSignedBy(transaction, serverAccountID)) {
      throw new _errors.InvalidSep10ChallengeError("Transaction not signed by server: '".concat(serverAccountID, "'"));
    }
    return {
      tx: transaction,
      clientAccountID: clientAccountID,
      matchedHomeDomain: matchedHomeDomain,
      memo: memo
    };
  }
  _Utils.readChallengeTx = readChallengeTx;
  function verifyChallengeTxThreshold(challengeTx, serverAccountID, networkPassphrase, threshold, signerSummary, homeDomains, webAuthDomain) {
    var signers = signerSummary.map(function (signer) {
      return signer.key;
    });
    var signersFound = verifyChallengeTxSigners(challengeTx, serverAccountID, networkPassphrase, signers, homeDomains, webAuthDomain);
    var weight = 0;
    var _loop = function _loop() {
      var _signerSummary$find;
      var signer = _signersFound[_i];
      var sigWeight = ((_signerSummary$find = signerSummary.find(function (s) {
        return s.key === signer;
      })) === null || _signerSummary$find === void 0 ? void 0 : _signerSummary$find.weight) || 0;
      weight += sigWeight;
    };
    for (var _i = 0, _signersFound = signersFound; _i < _signersFound.length; _i++) {
      _loop();
    }
    if (weight < threshold) {
      throw new _errors.InvalidSep10ChallengeError("signers with weight ".concat(weight, " do not meet threshold ").concat(threshold, "\""));
    }
    return signersFound;
  }
  _Utils.verifyChallengeTxThreshold = verifyChallengeTxThreshold;
  function verifyChallengeTxSigners(challengeTx, serverAccountID, networkPassphrase, signers, homeDomains, webAuthDomain) {
    var _readChallengeTx = readChallengeTx(challengeTx, serverAccountID, networkPassphrase, homeDomains, webAuthDomain),
      tx = _readChallengeTx.tx;
    var serverKP;
    try {
      serverKP = _jsStellarBase.Keypair.fromPublicKey(serverAccountID);
    } catch (err) {
      throw new Error("Couldn't infer keypair from the provided 'serverAccountID': " + err.message);
    }
    var clientSigners = new Set();
    var _iterator2 = _createForOfIteratorHelper(signers),
      _step2;
    try {
      for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
        var _signer = _step2.value;
        if (_signer === serverKP.publicKey()) {
          continue;
        }
        if (_signer.charAt(0) !== "G") {
          continue;
        }
        clientSigners.add(_signer);
      }
    } catch (err) {
      _iterator2.e(err);
    } finally {
      _iterator2.f();
    }
    if (clientSigners.size === 0) {
      throw new _errors.InvalidSep10ChallengeError("No verifiable client signers provided, at least one G... address must be provided");
    }
    var clientSigningKey;
    var _iterator3 = _createForOfIteratorHelper(tx.operations),
      _step3;
    try {
      for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
        var op = _step3.value;
        if (op.type === "manageData" && op.name === "client_domain") {
          if (clientSigningKey) {
            throw new _errors.InvalidSep10ChallengeError("Found more than one client_domain operation");
          }
          clientSigningKey = op.source;
        }
      }
    } catch (err) {
      _iterator3.e(err);
    } finally {
      _iterator3.f();
    }
    var allSigners = [serverKP.publicKey()].concat(_toConsumableArray(Array.from(clientSigners)));
    if (clientSigningKey) {
      allSigners.push(clientSigningKey);
    }
    var signersFound = gatherTxSigners(tx, allSigners);
    var serverSignatureFound = false;
    var clientSigningKeySignatureFound = false;
    for (var _i2 = 0, _signersFound2 = signersFound; _i2 < _signersFound2.length; _i2++) {
      var signer = _signersFound2[_i2];
      if (signer === serverKP.publicKey()) {
        serverSignatureFound = true;
      }
      if (signer === clientSigningKey) {
        clientSigningKeySignatureFound = true;
      }
    }
    if (!serverSignatureFound) {
      throw new _errors.InvalidSep10ChallengeError("Transaction not signed by server: '" + serverKP.publicKey() + "'");
    }
    if (clientSigningKey && !clientSigningKeySignatureFound) {
      throw new _errors.InvalidSep10ChallengeError("Transaction not signed by the source account of the 'client_domain' " + "ManageData operation");
    }
    if (signersFound.length === 1) {
      throw new _errors.InvalidSep10ChallengeError("None of the given signers match the transaction signatures");
    }
    if (signersFound.length !== tx.signatures.length) {
      throw new _errors.InvalidSep10ChallengeError("Transaction has unrecognized signatures");
    }
    signersFound.splice(signersFound.indexOf(serverKP.publicKey()), 1);
    if (clientSigningKey) {
      signersFound.splice(signersFound.indexOf(clientSigningKey), 1);
    }
    return signersFound;
  }
  _Utils.verifyChallengeTxSigners = verifyChallengeTxSigners;
  function verifyTxSignedBy(transaction, accountID) {
    return gatherTxSigners(transaction, [accountID]).length !== 0;
  }
  _Utils.verifyTxSignedBy = verifyTxSignedBy;
  function gatherTxSigners(transaction, signers) {
    var hashedSignatureBase = transaction.hash();
    var txSignatures = _toConsumableArray(transaction.signatures);
    var signersFound = new Set();
    var _iterator4 = _createForOfIteratorHelper(signers),
      _step4;
    try {
      for (_iterator4.s(); !(_step4 = _iterator4.n()).done;) {
        var signer = _step4.value;
        if (txSignatures.length === 0) {
          break;
        }
        var keypair = void 0;
        try {
          keypair = _jsStellarBase.Keypair.fromPublicKey(signer);
        } catch (err) {
          throw new _errors.InvalidSep10ChallengeError("Signer is not a valid address: " + err.message);
        }
        for (var i = 0; i < txSignatures.length; i++) {
          var decSig = txSignatures[i];
          if (!decSig.hint().equals(keypair.signatureHint())) {
            continue;
          }
          if (keypair.verify(hashedSignatureBase, decSig.signature())) {
            signersFound.add(signer);
            txSignatures.splice(i, 1);
            break;
          }
        }
      }
    } catch (err) {
      _iterator4.e(err);
    } finally {
      _iterator4.f();
    }
    return Array.from(signersFound);
  }
  _Utils.gatherTxSigners = gatherTxSigners;
  function validateTimebounds(transaction) {
    var gracePeriod = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    if (!transaction.timeBounds) {
      return false;
    }
    var now = Math.floor(Date.now() / 1000);
    var _transaction$timeBoun2 = transaction.timeBounds,
      minTime = _transaction$timeBoun2.minTime,
      maxTime = _transaction$timeBoun2.maxTime;
    return now >= Number.parseInt(minTime, 10) - gracePeriod && now <= Number.parseInt(maxTime, 10) + gracePeriod;
  }
})(Utils || (exports.Utils = Utils = {}));